@@ -1,4 +1,6 @@
--- Minetest: builtin/misc_register.lua
+-- Minetest: builtin/register.lua
+
+local S = core.get_translator("__builtin")
 
 --
 -- Make raw registration functions inaccessible to anyone except this file
@@ -326,7 +328,7 @@ end
 
 core.register_item(":unknown", {
 	type = "none",
-	description = "Unknown Item",
+	description = S("Unknown Item"),
 	inventory_image = "unknown_item.png",
 	on_place = core.item_place,
 	on_secondary_use = core.item_secondary_use,
@@ -336,7 +338,7 @@ core.register_item(":unknown", {
 })
 
 core.register_node(":air", {
-	description = "Air",
+	description = S("Air"),
 	inventory_image = "air.png",
 	wield_image = "air.png",
 	drawtype = "airlike",
@@ -353,7 +355,7 @@ core.register_node(":air", {
 })
 
 core.register_node(":ignore", {
-	description = "Ignore",
+	description = S("Ignore"),
 	inventory_image = "ignore.png",
 	wield_image = "ignore.png",
 	drawtype = "airlike",
@@ -366,11 +368,12 @@ core.register_node(":ignore", {
 	air_equivalent = true,
 	drop = "",
 	groups = {not_in_creative_inventory=1},
+	node_placement_prediction = "",
 	on_place = function(itemstack, placer, pointed_thing)
 		core.chat_send_player(
 				placer:get_player_name(),
 				core.colorize("#FF0000",
-				"You can't place 'ignore' nodes!"))
+				S("You can't place 'ignore' nodes!")))
 		return ""
 	end,
 })
@@ -400,8 +403,14 @@ function core.override_item(name, redefinition)
 	register_item_raw(item)
 end
 
-
-core.callback_origins = {}
+do
+	local default = {mod = "??", name = "??"}
+	core.callback_origins = setmetatable({}, {
+		__index = function()
+			return default
+		end
+	})
+end
 
 function core.run_callbacks(callbacks, mode, ...)
 	assert(type(callbacks) == "table")
@@ -416,9 +425,7 @@ function core.run_callbacks(callbacks, mode, ...)
 	local ret = nil
 	for i = 1, cb_len do
 		local origin = core.callback_origins[callbacks[i]]
-		if origin then
-			core.set_last_run_mod(origin.mod)
-		end
+		core.set_last_run_mod(origin.mod)
 		local cb_ret = callbacks[i](...)
 
 		if mode == 0 and i == 1 then
@@ -607,6 +614,7 @@ core.registered_on_modchannel_message, core.register_on_modchannel_message = mak
 core.registered_on_player_inventory_actions, core.register_on_player_inventory_action = make_registration()
 core.registered_allow_player_inventory_actions, core.register_allow_player_inventory_action = make_registration()
 core.registered_on_rightclickplayers, core.register_on_rightclickplayer = make_registration()
+core.registered_on_liquid_transformed, core.register_on_liquid_transformed = make_registration()
 
 --
 -- Compatibility for on_mapgen_init()
