@@ -1,14 +1,51 @@
+-- creative/init.lua
+
+-- Load support for MT game translation.
+local S = minetest.get_translator("creative")
+
 creative = {}
+creative.get_translator = S
 
-local creative_mode_cache = minetest.settings:get_bool("creative_mode")
+local function update_sfinv(name)
+	minetest.after(0, function()
+		local player = minetest.get_player_by_name(name)
+		if player then
+			if sfinv.get_page(player):sub(1, 9) == "creative:" then
+				sfinv.set_page(player, sfinv.get_homepage_name(player))
+			else
+				sfinv.set_player_inventory_formspec(player)
+			end
+		end
+	end)
+end
 
+minetest.register_privilege("creative", {
+	description = S("Allow player to use creative inventory"),
+	give_to_singleplayer = false,
+	give_to_admin = false,
+	on_grant = update_sfinv,
+	on_revoke = update_sfinv,
+})
+
+-- Override the engine's creative mode function
+local old_is_creative_enabled = minetest.is_creative_enabled
+
+function minetest.is_creative_enabled(name)
+	if name == "" then
+		return old_is_creative_enabled(name)
+	end
+	return minetest.check_player_privs(name, {creative = true}) or
+		old_is_creative_enabled(name)
+end
+
+-- For backwards compatibility:
 function creative.is_enabled_for(name)
-	return creative_mode_cache
+	return minetest.is_creative_enabled(name)
 end
 
 dofile(minetest.get_modpath("creative") .. "/inventory.lua")
 
-if creative_mode_cache then
+if minetest.is_creative_enabled("") then
 	-- Dig time is modified according to difference (leveldiff) between tool
 	-- 'maxlevel' and node 'level'. Digtime is divided by the larger of
 	-- leveldiff and 1.
@@ -18,10 +55,8 @@ if creative_mode_cache then
 	local digtime = 42
 	local caps = {times = {digtime, digtime, digtime}, uses = 0, maxlevel = 256}
 
-	minetest.register_item(":", {
-		type = "none",
-		wield_image = "wieldhand.png",
-		wield_scale = {x = 1, y = 1, z = 2.5},
+	-- Override the hand tool
+	minetest.override_item("", {
 		range = 10,
 		tool_capabilities = {
 			full_punch_interval = 0.5,
@@ -32,6 +67,9 @@ if creative_mode_cache then
 				snappy  = caps,
 				choppy  = caps,
 				oddly_breakable_by_hand = caps,
+				-- dig_immediate group doesn't use value 1. Value 3 is instant dig
+				dig_immediate =
+					{times = {[2] = digtime, [3] = 0}, uses = 0, maxlevel = 256},
 			},
 			damage_groups = {fleshy = 10},
 		}
@@ -41,7 +79,7 @@ end
 -- Unlimited node placement
 minetest.register_on_placenode(function(pos, newnode, placer, oldnode, itemstack)
 	if placer and placer:is_player() then
-		return creative.is_enabled_for(placer:get_player_name())
+		return minetest.is_creative_enabled(placer:get_player_name())
 	end
 end)
 
@@ -49,7 +87,7 @@ end)
 local old_handle_node_drops = minetest.handle_node_drops
 function minetest.handle_node_drops(pos, drops, digger)
 	if not digger or not digger:is_player() or
-		not creative.is_enabled_for(digger:get_player_name()) then
+		not minetest.is_creative_enabled(digger:get_player_name()) then
 		return old_handle_node_drops(pos, drops, digger)
 	end
 	local inv = digger:get_inventory()
