@@ -3,6 +3,52 @@
 -- support for MT game translation.
 local S = default.get_translator
 
+local esc = minetest.formspec_escape
+local formspec_size = "size[8,8]"
+
+local function formspec_core(tab)
+	if tab == nil then tab = 1 else tab = tostring(tab) end
+	return "tabheader[0,0;book_header;" ..
+		esc(S("Write")) .. "," ..
+		esc(S("Read")) .. ";" ..
+		tab  ..  ";false;false]"
+end
+
+local function formspec_write(title, text)
+	return "field[0.5,1;7.5,0;title;" .. esc(S("Title:")) .. ";" ..
+			esc(title) .. "]" ..
+		"textarea[0.5,1.5;7.5,7;text;" .. esc(S("Contents:")) .. ";" ..
+			esc(text) .. "]" ..
+		"button_exit[2.5,7.5;3,1;save;" .. esc(S("Save")) .. "]"
+end
+
+local function formspec_read(owner, title, string, text, page, page_max)
+	return "label[0.5,0.5;" .. esc(S("by @1", owner)) .. "]" ..
+		"tablecolumns[color;text]" ..
+		"tableoptions[background=#00000000;highlight=#00000000;border=false]" ..
+		"table[0.4,0;7,0.5;title;#FFFF00," .. esc(title) .. "]" ..
+		"textarea[0.5,1.5;7.5,7;;" ..
+			esc(string ~= "" and string or text) .. ";]" ..
+		"button[2.4,7.6;0.8,0.8;book_prev;<]" ..
+		"label[3.2,7.7;" .. esc(S("Page @1 of @2", page, page_max)) .. "]" ..
+		"button[4.9,7.6;0.8,0.8;book_next;>]"
+end
+
+local function formspec_string(lpp, page, lines, string)
+	for i = ((lpp * page) - lpp) + 1, lpp * page do
+		if not lines[i] then break end
+		string = string .. lines[i] .. "\n"
+	end
+	return string
+end
+
+local book_writers = {}
+
+minetest.register_on_leaveplayer(function(player)
+	book_writers[player:get_player_name()] = nil
+end)
+
+local tab_number
 local lpp = 14 -- Lines per book's page
 local function book_on_use(itemstack, user)
 	local player_name = user:get_player_name()
@@ -19,8 +65,8 @@ local function book_on_use(itemstack, user)
 	local data = meta:to_table().fields
 
 	if data.owner then
-		title = data.title
-		text = data.text
+		title = data.title or ""
+		text = data.text or ""
 		owner = data.owner
 
 		for str in (text .. "\n"):gmatch("([^\n]*)[\n]") do
@@ -30,37 +76,28 @@ local function book_on_use(itemstack, user)
 		if data.page then
 			page = data.page
 			page_max = data.page_max
-
-			for i = ((lpp * page) - lpp) + 1, lpp * page do
-				if not lines[i] then break end
-				string = string .. lines[i] .. "\n"
-			end
+			string = formspec_string(lpp, page, lines, string)
 		end
 	end
 
 	local formspec
-	local esc = minetest.formspec_escape
-	if owner == player_name then
-		formspec = "size[8,8]" ..
-			"field[0.5,1;7.5,0;title;" .. esc(S("Title:")) .. ";" ..
-				esc(title) .. "]" ..
-			"textarea[0.5,1.5;7.5,7;text;" .. esc(S("Contents:")) .. ";" ..
-				esc(text) .. "]" ..
-			"button_exit[2.5,7.5;3,1;save;" .. esc(S("Save")) .. "]"
+	if title == "" and text == "" then
+		formspec = formspec_write(title, text)
+	elseif owner == player_name then
+		local tab = tab_number or 1
+		if tab == 2 then
+			formspec = formspec_core(tab) ..
+				formspec_read(owner, title, string, text, page, page_max)
+		else
+			formspec = formspec_core(tab) .. formspec_write(title, text)
+		end
 	else
-		formspec = "size[8,8]" ..
-			"label[0.5,0.5;" .. esc(S("by @1", owner)) .. "]" ..
-			"tablecolumns[color;text]" ..
-			"tableoptions[background=#00000000;highlight=#00000000;border=false]" ..
-			"table[0.4,0;7,0.5;title;#FFFF00," .. esc(title) .. "]" ..
-			"textarea[0.5,1.5;7.5,7;;" ..
-				minetest.formspec_escape(string ~= "" and string or text) .. ";]" ..
-			"button[2.4,7.6;0.8,0.8;book_prev;<]" ..
-			"label[3.2,7.7;" .. esc(S("Page @1 of @2", page, page_max)) .. "]" ..
-			"button[4.9,7.6;0.8,0.8;book_next;>]"
+		formspec = formspec_read(owner, title, string, text, page, page_max)
 	end
 
-	minetest.show_formspec(player_name, "default:book", formspec)
+	minetest.show_formspec(player_name, "default:book", formspec_size .. formspec)
+	-- Store the wield index in case the user accidentally switches before the formspec is shown
+	book_writers[player_name] = {wield_index = user:get_wield_index()}
 	return itemstack
 end
 
@@ -68,14 +105,56 @@ local max_text_size = 10000
 local max_title_size = 80
 local short_title_size = 35
 minetest.register_on_player_receive_fields(function(player, formname, fields)
-	if formname ~= "default:book" then return end
+	if formname ~= "default:book" then
+		return
+	end
+	local player_name = player:get_player_name()
 	local inv = player:get_inventory()
-	local stack = player:get_wielded_item()
+	if not book_writers[player_name] then
+		return
+	end
+	local wield_index = book_writers[player_name].wield_index
+	local wield_list = player:get_wield_list()
+	local stack = inv:get_stack(wield_list, wield_index)
+	local written = stack:get_name() == "default:book_written"
+	if stack:get_name() ~= "default:book" and not written then
+		-- No book in the wield slot, abort & inform the player
+		minetest.chat_send_player(player_name, S("The book you were writing to mysteriously disappeared."))
+		return
+	end
+	local data = stack:get_meta():to_table().fields
 
-	if fields.save and fields.title and fields.text
-			and fields.title ~= "" and fields.text ~= "" then
-		local new_stack, data
-		if stack:get_name() ~= "default:book_written" then
+	local title = data.title or ""
+	local text = data.text or ""
+
+	if fields.book_header ~= nil and data.owner == player_name then
+		local contents
+		local tab = tonumber(fields.book_header)
+		if tab == 1 then
+			contents = formspec_core(tab) ..
+				formspec_write(title, text)
+		elseif tab == 2 then
+			local lines, string = {}, ""
+			for str in (text .. "\n"):gmatch("([^\n]*)[\n]") do
+				lines[#lines+1] = str
+			end
+			string = formspec_string(lpp, data.page, lines, string)
+			contents = formspec_read(player_name, title, string,
+				text, data.page, data.page_max)
+		end
+		tab_number = tab
+		local formspec = formspec_size .. formspec_core(tab) .. contents
+		minetest.show_formspec(player_name, "default:book", formspec)
+		return
+	end
+
+	if fields.close then
+		book_writers[player_name] = nil
+	end
+
+	if fields.save and fields.title and fields.text then
+		local new_stack
+		if not written then
 			local count = stack:get_count()
 			if count == 1 then
 				stack:set_name("default:book_written")
@@ -83,11 +162,9 @@ minetest.register_on_player_receive_fields(function(player, formname, fields)
 				stack:set_count(count - 1)
 				new_stack = ItemStack("default:book_written")
 			end
-		else
-			data = stack:get_meta():to_table().fields
 		end
 
-		if data and data.owner and data.owner ~= player:get_player_name() then
+		if data.owner ~= player_name and title ~= "" and text ~= "" then
 			return
 		end
 
@@ -117,8 +194,7 @@ minetest.register_on_player_receive_fields(function(player, formname, fields)
 		end
 
 	elseif fields.book_next or fields.book_prev then
-		local data = stack:get_meta():to_table().fields
-		if not data or not data.page then
+		if not data.page then
 			return
 		end
 
@@ -142,64 +218,9 @@ minetest.register_on_player_receive_fields(function(player, formname, fields)
 	end
 
 	-- Update stack
-	player:set_wielded_item(stack)
+	inv:set_stack(wield_list, wield_index, stack)
 end)
 
-minetest.register_craftitem("default:skeleton_key", {
-	description = S("Skeleton Key"),
-	inventory_image = "default_key_skeleton.png",
-	on_use = function(itemstack, user, pointed_thing)
-		if pointed_thing.type ~= "node" then
-			return itemstack
-		end
-
-		local pos = pointed_thing.under
-		local node = minetest.get_node(pos)
-
-		if not node then
-			return itemstack
-		end
-
-		local node_reg = minetest.registered_nodes[node.name]
-		local on_skeleton_key_use = node_reg and node_reg.on_skeleton_key_use
-		if not on_skeleton_key_use then
-			return itemstack
-		end
-
-		-- make a new key secret in case the node callback needs it
-		local random = math.random
-		local newsecret = string.format(
-			"%04x%04x%04x%04x",
-			random(2^16) - 1, random(2^16) - 1,
-			random(2^16) - 1, random(2^16) - 1)
-
-		local secret, _, _ = on_skeleton_key_use(pos, user, newsecret)
-
-		if secret then
-			local inv = minetest.get_inventory({type="player", name=user:get_player_name()})
-
-			-- update original itemstack
-			itemstack:take_item()
-
-			-- finish and return the new key
-			local new_stack = ItemStack("default:key")
-			local meta = new_stack:get_meta()
-			meta:set_string("secret", secret)
-			meta:set_string("description", S("Key to @1's @2", user:get_player_name(),
-				minetest.registered_nodes[node.name].description))
-
-			if itemstack:get_count() == 0 then
-				itemstack = new_stack
-			else
-				if inv:add_item("main", new_stack):get_count() > 0 then
-					minetest.add_item(user:get_pos(), new_stack)
-				end -- else: added to inventory successfully
-			end
-
-			return itemstack
-		end
-	end
-})
 
 --
 -- Craftitem registry
@@ -428,13 +449,6 @@ minetest.register_craft({
 	}
 })
 
-minetest.register_craft({
-	output = "default:skeleton_key",
-	recipe = {
-		{"default:gold_ingot"},
-	}
-})
-
 minetest.register_craft({
 	output = "default:steel_ingot 9",
 	recipe = {
@@ -478,19 +492,6 @@ minetest.register_craft({
 	recipe = "default:gold_lump",
 })
 
-minetest.register_craft({
-	type = "cooking",
-	output = "default:gold_ingot",
-	recipe = "default:key",
-	cooktime = 5,
-})
-
-minetest.register_craft({
-	type = "cooking",
-	output = "default:gold_ingot",
-	recipe = "default:skeleton_key",
-	cooktime = 5,
-})
 
 minetest.register_craft({
 	type = "cooking",
